use crate::{decks::new_blackjack_shoe, hand::Hand};

pub struct BlackjackGame {
    shoe: Hand,
    player_hand: Hand,
    dealer_hand: Hand,
}

pub enum DealResult {
    DealerBlackJack, // Dealer has a blackjack. Player does not.
    Push,            // Dealer has a Blackjack and so does the Player
    PlayerBlackJack, // Player has a blackjack. Dealer does not.
    Continue,        // Game continues
}

pub enum HitResult {
    Bust,
    Continue,
}

pub enum DealerPlayResult {
    Bust,
    Push,            // Player and dealer have the same value
    StandPlayerLose, // Dealer stands and wins
    StandPlayerWin,  // Dealer stands and loses
}

impl BlackjackGame {
    pub fn new() -> Self {
        BlackjackGame {
            shoe: new_blackjack_shoe(6),
            player_hand: Hand::new(),
            dealer_hand: Hand::new(),
        }
    }

    pub fn get_dealer_hand(&self) -> &Hand {
        &self.dealer_hand
    }

    pub fn get_player_hand(&self) -> &Hand {
        &self.player_hand
    }

    pub fn deal(&mut self) -> DealResult {
        self.player_hand.add_card(self.shoe.pop_card().unwrap());
        self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

        self.player_hand.add_card(self.shoe.pop_card().unwrap());
        self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

        if self.dealer_hand.is_backjack() {
            if self.player_hand.is_backjack() {
                return DealResult::Push;
            }

            return DealResult::DealerBlackJack;
        }

        if self.player_hand.is_backjack() {
            return DealResult::PlayerBlackJack;
        }

        DealResult::Continue
    }

    pub fn hit(&mut self) -> HitResult {
        self.player_hand.add_card(self.shoe.pop_card().unwrap());

        if self.player_hand.get_blackjack_value() > 21 {
            return HitResult::Bust;
        }

        HitResult::Continue
    }

    pub fn stand(&mut self) {
        // ???
    }

    pub fn dealer_play(&mut self) -> DealerPlayResult {
        // Stand on 17 or above
        // Hit on 16 and below

        loop {
            let dealer_value = self.dealer_hand.get_blackjack_value();
            if dealer_value >= 17 {
                let player_value = self.player_hand.get_blackjack_value();

                return match dealer_value.cmp(&player_value) {
                    std::cmp::Ordering::Equal => DealerPlayResult::Push,
                    std::cmp::Ordering::Greater => DealerPlayResult::StandPlayerLose,
                    std::cmp::Ordering::Less => DealerPlayResult::StandPlayerWin,
                };
            }

            self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

            if self.dealer_hand.get_blackjack_value() > 21 {
                return DealerPlayResult::Bust;
            }
        }
    }
}
